# skill_tree_ai

AI 技能树 是[技能森林](https://gitcode.net/csdn/skill_tree)的一部分。

## 初始化技能树

技能树合成和id生成脚本目前用 Python 脚本统一处理

```bash
pip install -r requirement.txt
```


## 目录结构说明

data目录下包含 难度节点/章节点/知识节点 3级目录结构

* 技能树`骨架文件`：
    * 位置：`data/tree.json` 
    * 说明：该文件是执行 `python main.py` 生成的，请勿人工编辑
* 技能树`根节点`配置文件：
    * 位置：`data/config.json`
    * 说明：可编辑配置关键词等字段，其中 `node_id` 字段是生成的，请勿编辑
* 技能树`难度节点`：
    * 位置：`data/xxx`，例如: `data/1.AI初阶`
    * 说明：
        * 每个技能树有 3 个等级，目录前的序号是**必要**的，用来保持文件夹目录的顺序
        * 每个目录下有一个 `config.json` 可配置关键词信息，其中 `node_id` 字段是生成的，请勿编辑
* 技能树`章节点`：
    * 位置：`data/xxx/xxx`，例如：`data/1.AI初阶/1.预备知识`
    * 说明：
        * 每个技能树的每个难度等级有 n 个章节，目录前的序号是**必要**的，用来保持文件夹目录的顺序
        * 每个目录下有一个 `config.json` 可配置关键词信息，其中 `node_id` 字段是生成的，请勿编辑
* 技能树`知识节点`：
    * 位置：`data/xxx/xxx`，例如：`data/1.AI初阶/1.预备知识/1.AI简介`
    * 说明：
        * 每个技能树的每章有 n 个知识节点，目录前的序号是必要的，用来保持文件夹目录的顺序
        * 每个目录下有一个 `config.json`
            * 其中 `node_id` 字段是生成的，请勿编辑
            * 其中 `keywords` 可配置关键字字段
            * 其中 `children` 可配置该`知识节点`下的子树结构信息，参考后面描述
            * 其中 `export` 可配置该`知识节点`下的导出习题信息，参考后面描述


## `知识节点` 子树信息结构

例如 `data/1.AI初阶/1.预备知识/1.AI简介/config.json` 里配置对该知识节点子树信息结构，用来增加技能树社区服务在该知识节点上的深度数据匹配：

```json
{
    // ...

    "children": [
        {
            "AI简史": {
                "keywords": [
                    "AI起源",
                    "人工智能简史"
                ],
                "children": []
            }
        }
    ],
}
```


## `知识节点` 的导出习题编辑

例如 `data/1.AI初阶/1.预备知识/1.AI简介/config.json` 里配置对该知识节点导出的习题

```json
{
    // ...
    "export": [
        "helloworld.json",
        // ...
    ]
}
```

在 `export` 字段中，我们列出习题定义的`json`文件列表 ，下面我们了解如何编写习题。

## `知识节点` 的导出习题选项配置编辑

目前我们支持使用 markdown 语法直接编辑习题和各选项。

如前文内容，我们在知识节点下增加习题 `helloworld`的定义文件，即在`data/1.AI初阶/1.预备知识/1.AI简介` 目录增加一个`helloworld.json`文件：

```json
{
    "type": "code_options",
    "author": "幻灰龙",
    "source": "helloworld.md",
    "notebook_enable": true
}
```

其中

* `type` 字段目前都固定是 `code_options`，
* `notebook_enable` 字段决定这个习题是否生成对应的 `notebook`
* `source` 字段代表习题编辑的 `markdwon` 文件。

现在我们新建一个 `helloworld.md` 并编辑为：

````markdown
# Hello World

HelloWorld, 请阅读如下代码：

```python
import numpy as np

def test():
    X = np.array([[1, 1], [1, 2], [2, 2], [2, 3]])
    y = np.dot(X, np.array([1, 2])) + 3

    // TODO(选择选项中的代码填充此处)

    y_predict = reg.predict(np.array([[3, 5]]))
    print(y_predict)

if __name__ == '__main__':
    test()
```

若将以下选项中的代码分别填充到上述代码中**TODO**处，哪个选项不是线性模型？

## template

```java
import numpy as np

def test():
    X = np.array([[1, 1], [1, 2], [2, 2], [2, 3]])
    y = np.dot(X, np.array([1, 2])) + 3

    // 下面的 code 占位符会被替换成答案和选项代码
    $code

    y_predict = reg.predict(np.array([[3, 5]]))
    print(y_predict)

if __name__ == '__main__':
    test()
```


## 答案

```python
from sklearn import svm
reg = svm.SVC(kernel='rbf').fit(X, y)
```

## 选项

### 使用 LinearRegression

```python
from sklearn.linear_model import LinearRegression
reg = LinearRegression().fit(X, y)
```

### 使用岭回归

```python
from sklearn.linear_model import Ridge
reg = Ridge(alpha=0.1)
```

### 使用拉索算法

```python
from sklearn.linear_model import Lasso
reg = Lasso(alpha=0.1).fit(X, y)
```
````

这是一个最基本的习题MarkDown结构，说明如下：

* 一级标题是`习题标题`
* 一级标题紧接着的段落是`习题描述`
* `## template` 是用于和答案、选项结合合成`NoteBook`代码用的模版
* `## 答案` 是习题选项中符合题目描述的答案项
* `## 选项` 下包含几个混淆用的选项
    * 每个选项带有一个三级标题，例如`### 使用 LinearRegression`，
    * 最终生成的习题中不包含选项的三级标题，所以这个标题可以用来标注一些编辑信息

## 可选的习题源代码项目

编辑习题中，为了测试方便，可以直接在3级知识节点目录下创建对应的习题代码子目录



## 技能树合成

在根目录下执行 `python main.py` 会合成技能树文件，合成的技能树文件: `data/tree.json`

* 合成过程中，会自动检查每个目录下 `config.json` 里的 `node_id` 是否存在，不存在则生成
* 合成过程中，会自动检查每个知识点目录下 `config.json` 里的 `export` 里导出的习题配置，检查是否存在`exercise_id` 字段，如果不存在则生成

## FAQ

**难度目录是固定的么？**

1. data/xxx 目录下的子目录是固定的初/中/高三个难度等级目录

**如何增加章目录？**

1. 在VSCode里打开项目仓库
2. 在对应的难度等级目录新建章目录，例如在 data/1.xxx初阶/ 下新建章文件夹，data/1.xxx初阶/1.yyy
3. 在项目根目录下执行 python main.py 脚本，会自动生成章的配置文件 data/1.xxx初阶/1.yyy/config.json

**如何增加节目录?**:
1. 直接在VSCode里创建文件夹，例如 "data/1.xxx初阶/1.yyy/2.zzz"
2. 项目根目录下执行 python main.py 会自动为新增节创建配置文件 data/1.xxx初阶/1.yyy/2.zzz/config.json

**如何在节下新增一个习题**:
3. 在"data/1.xxx初阶/1.yyy/2.zzz" 目录下添加一个 markdown 文件编辑，例如 yyy.md，按照习题markdown格式编辑习题。
4. md编辑完后，可以再次执行  python main.py 会自动生成同名的 yyy.json，并将 yyy.json 添加到config.json 的export数组里。
5. yyy.json里的author信息放作者 CSDN ID。
